#pragma once
#ifndef ECHO_SERVER_TEST_I_TEST_MESSAGE_INCLUDED__
#define ECHO_SERVER_TEST_I_TEST_MESSAGE_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ITestMessage.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include <wtypes.h>

#include "JetByteTools/CoreTools/tstring.h"

#include <vector>

///////////////////////////////////////////////////////////////////////////////
// ITestMessage
///////////////////////////////////////////////////////////////////////////////

class ITestMessage
{
   public :

      typedef unsigned long DataLength;

      class RoundTripTimes
      {
         public :

         RoundTripTimes()
               :  sentTime(0),
                  recvTime(0)
            {
            }

            DWORD sentTime;
            DWORD recvTime;

         DWORD GetRoundTripTime() const
            {
               return (recvTime >= sentTime ? recvTime - sentTime : MAXDWORD - (sentTime - recvTime));
            }
      };

      typedef std::vector<RoundTripTimes> Times;

      virtual ~ITestMessage() = default;

      virtual void ExpectRepeats(
         size_t numberOfRepeats) = 0;

      virtual JetByteTools::Core::_tstring GetIdentifier() const = 0;

      virtual const Times &GetTimes() const = 0;

      virtual const BYTE *GetBytes() const = 0;

      virtual DataLength GetSize() const = 0;

      virtual bool ValidateResponse(
         const BYTE *pData,
         DataLength dataLength,
         DataLength &dataUsed) const = 0;

      virtual DataLength GetRecvOffset() const = 0;

      virtual DataLength GetResponseSize() const = 0;

      virtual bool BytesSent(
         DataLength bytesSent) = 0;

      virtual void ResetMessage() = 0;
};

#endif // ECHO_SERVER_TEST_I_TEST_MESSAGE_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ITestMessage.h
///////////////////////////////////////////////////////////////////////////////
