#pragma once
#ifndef ECHO_SERVER_TEST_MESSAGE_COLLECTION_INCLUDED__
#define ECHO_SERVER_TEST_MESSAGE_COLLECTION_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: MessageCollection.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/WaitableCounter.h"

#include <vector>
#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CTestMessages;

///////////////////////////////////////////////////////////////////////////////
// CMessageCollection
///////////////////////////////////////////////////////////////////////////////

class CMessageCollection
{
   private :

      typedef std::vector<CTestMessages *> Messages;

   public :

      explicit CMessageCollection(
         long numberOfConnections);

      CMessageCollection(
         const CMessageCollection &rhs) = default;

      CMessageCollection &operator=(
         const CMessageCollection &rhs) = default;

      ~CMessageCollection();

      void Add(
         CTestMessages *pMessages);

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      bool CheckMessages(
         StringErrors &errors);

      HANDLE GetConnectionsCompleteEvent();

      HANDLE GetMessagesCompleteEvent();

      void MessagesComplete();

      void ConnectionComplete(
         bool succeeded);

      long ConnectionsEstablished() const;

      long ConnectionsFailed() const;

      long GetNumberOfConnectionsWithMessagesPending() const;

      typedef Messages::const_iterator Iterator;

      Iterator Begin() const;

      Iterator End() const;

      DWORD GetAverageRTT() const;

      DWORD GetMaxRTT() const;

      DWORD GetMinRTT() const;

   private :

      Messages m_messages;

      const long m_numberOfConnections;

      volatile long m_connectionsEstablished;

      JetByteTools::Core::CWaitableCounter m_connectionCounter;
      JetByteTools::Core::CWaitableCounter m_messageCounter;
};

#endif // ECHO_SERVER_TEST_MESSAGE_COLLECTION_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: MessageCollection.h
///////////////////////////////////////////////////////////////////////////////
