#pragma once
#ifndef SCHANNEL_SERVER_TEST_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
#define SCHANNEL_SERVER_TEST_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SChannelTools/StreamSocketConnectionManager.h"

#include "JetByteTools/SChannelTools/StreamSocketConnectionManagerCallback.h"

#include "ServerCommon/ThreadedCallbackTimerQueue.h"

#include "JetByteTools/CoreTools/LockableObject.h"

#include "JetByteTools/SocketTools/ReadTimeoutStreamSocketConnectionFilter.h"

#include <map>

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

class CTestMessages;

///////////////////////////////////////////////////////////////////////////////
// CStreamSocketConnectionManager
///////////////////////////////////////////////////////////////////////////////

class CStreamSocketConnectionManager :
   public JetByteTools::SSPI::SChannel::CStreamSocketConnectionManager,
   private JetByteTools::Core::IQueueTimers::Timer,
   private JetByteTools::SSPI::SChannel::CStreamSocketConnectionManagerCallback,
   private JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter::TimerCallback
{
   public :

      CStreamSocketConnectionManager(
         JetByteTools::SSPI::SChannel::CCredentials &credentials,
         JetByteTools::Milliseconds readTimeout,
         bool ignoreValidationErrors,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator);

      CStreamSocketConnectionManager(
         const CStreamSocketConnectionManager &rhs) = default;

      ~CStreamSocketConnectionManager();

      CStreamSocketConnectionManager &operator=(
         const CStreamSocketConnectionManager &rhs) = default;

      bool DumpErrors() const;

   private :

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnPreOutgoingConnect(
         JetByteTools::Core::IIndexedOpaqueUserData &userData,
         const JetByteTools::Socket::IAddress &address,
         const void *pUserData) override;

      void OnSecureConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnOutgoingConnectionFailed(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address,
         DWORD lastError) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnSecureSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void FatalError(
         JetByteTools::Socket::IStreamSocket &socket,
         SECURITY_STATUS status) override;

      void OnContextComplete(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::SSPI::SChannel::CContext &context) override;

      bool Verify(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Core::_tstring &targetName,
         const JetByteTools::SSPI::SChannel::CContext &context) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionReset(
          JetByteTools::Socket::IStreamSocket &socket,
          DWORD lastError) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnConnectionClosure(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::Socket::ConnectionClosureReason reason) override;

      // Implement IQueueTimers::Timer

      void OnTimer(
         JetByteTools::Core::IQueueTimers::Timer::UserData userData) override;

      // Implement CReadTimeoutStreamSocketConnectionFilter::TimerCallback

      void OnTimer(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter::TimerCallback::UserData userData) override;

      void ProcessData(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      void SendData(
         JetByteTools::Socket::IStreamSocket &socket);

      void SetSendTimer(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::Milliseconds timeout);

      void DisplayCertChain(
         const CERT_CONTEXT *pServerCert);

      void DisplayWinVerifyTrustError(
         HRESULT status);

      HRESULT VerifyServerCertificate(
         const CERT_CONTEXT *pServerCert,
         const JetByteTools::Core::_tstring &serverName);

      void OnCertificateError(
         const JetByteTools::Core::_tstring &message);

      CThreadedCallbackTimerQueue m_timerQueue;

      JetByteTools::Core::CLockableObject m_lock;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_timerHandleIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_userDataIndex;

      typedef std::map<JetByteTools::Core::_tstring, size_t> StringErrors;

      typedef std::map<DWORD, size_t> LastErrors;

      LastErrors m_connectionErrors;
      LastErrors m_connectionResetErrors;

      StringErrors m_errors;

      StringErrors m_certificateErrors;

      bool DumpErrors(
         const JetByteTools::Core::_tstring &errorType,
         const LastErrors &lastErrors) const;

      JetByteTools::Socket::CReadTimeoutStreamSocketConnectionFilter m_timeoutFilter;

      const JetByteTools::Milliseconds m_readTimeout;

      const bool m_ignoreValidationErrors;
};

#endif // SCHANNEL_SERVER_TEST_STREAM_SOCKET_CONNECTION_MANAGER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: StreamSocketConnectionManager.h
///////////////////////////////////////////////////////////////////////////////
