#pragma once
#ifndef PER_CONNECTION_DATA_INCLUDED__
#define PER_CONNECTION_DATA_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: PerConnectionData.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2016 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "MessageAccumulator.h"

#include "JetByteTools/CoreTools/LockableObject.h"

#include "JetByteTools/IOTools/SortedBufferProcessor.h"
#include "JetByteTools/IOTools/BufferChain.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace IO
   {
   }

   namespace Socket
   {
      class IStreamSocket;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CPerConnectionData
///////////////////////////////////////////////////////////////////////////////

class CPerConnectionData : public CMessageAccumulator
{
   public :

      CPerConnectionData(
         JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex bufferChainIndex,
         JetByteTools::IO::IBuffer::BufferSize maxMessageLength,
         bool useMultipleWrites);

      CPerConnectionData(
         const CPerConnectionData &rhs) = delete;

      CPerConnectionData &operator=(
         const CPerConnectionData &rhs) = delete;

      void ProcessMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

   private :

      void InternalProcessMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

      void InternalProcessMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CBufferChain &buffers);

      JetByteTools::Core::CLockableObject m_lock;

      JetByteTools::IO::CSortedBufferProcessor m_outOfSequenceReads;

      const bool m_useMultipleWrites;

      const bool m_writeAsBytes;

      enum ProcessHow
      {
         ProcessContiguously,
         ProcessBufferChain,
         ProcessAlternate
      };

      ProcessHow m_processHow;

      bool m_processContiguouslyThisTime;

      bool m_useMultipleWritesThisTime;

      bool m_writeAsBytesThisTime;
};

#endif // PER_CONNECTION_DATA_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: PerConnectionData.h
///////////////////////////////////////////////////////////////////////////////
