#pragma once
#ifndef SCHANNEL_ECHO_SERVER_SOCKET_SERVER_INCLUDED__
#define SCHANNEL_ECHO_SERVER_SOCKET_SERVER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2008 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/WinsockWrapper.h"

#include "JetByteTools/Win32Tools/CriticalSection.h"
#include "JetByteTools/CoreTools/tstring.h"

#include "JetByteTools/IOTools/BufferAllocator.h"

#include "JetByteTools/SChannelTools/StreamSocketServer.h"

#include "JetByteTools/SChannelTools/StreamSocketServerCallback.h"

#include "ThreadPool.h"

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::SSPI::SChannel::CStreamSocketServer,
   private JetByteTools::SSPI::SChannel::CStreamSocketServerCallback
{
   public :

      CSocketServer(
         CThreadPool &businessLogicPool,
         JetByteTools::IO::IBuffer::BufferSize maxMessageLength,
         bool useMultipleWrites,
         JetByteTools::SSPI::SChannel::CCredentials &credentials,
         bool verifyPeer,
         bool allowUntrustedRoot,
         bool ignoreValidationErrors,
         const JetByteTools::Socket::IFullAddress &address,
         JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::SocketBufferSize recvBufferSize,
         JetByteTools::Socket::SocketBufferSize sendBufferSize,
         JetByteTools::Socket::ZeroByteReadConfiguration zeroByteReadConfiguration);

      CSocketServer(
         const CSocketServer &rhs) = delete;

      ~CSocketServer();

      CSocketServer &operator=(
         const CSocketServer &rhs) = delete;

      void ProcessMessageOnWorkerThread(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer,
         bool afterShutdown) const;

   private :

      typedef JetByteTools::SSPI::SChannel::CStreamSocketServer Base;

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnSecureConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnSecureSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnContextComplete(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::SSPI::SChannel::CContext &context) override;

      bool Verify(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Core::_tstring &targetName,
         const JetByteTools::SSPI::SChannel::CContext &context) override;

      void FatalError(
         JetByteTools::Socket::IStreamSocket &socket,
         SECURITY_STATUS status) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void ProcessDataStream(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      const JetByteTools::IO::IBuffer::BufferSize m_maxMessageLength;

      const bool m_useMultipleWrites;

      CThreadPool &m_pool;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_perConnectionIndex;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_bufferChainIndex;

      const bool m_allowUntrustedRoot;
      const bool m_ignoreValidationErrors;
};

#endif // SCHANNEL_ECHO_SERVER_SOCKET_SERVER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
