#pragma once
#ifndef OPEN_SSL_ECHO_SERVER_THREAD_POOL_INCLUDED__
#define OPEN_SSL_ECHO_SERVER_THREAD_POOL_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: ThreadPool.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/WinsockWrapper.h"

#include "JetByteTools/CoreTools/Types.h"
#include "JetByteTools/CoreTools/ThreadPool.h"
#include "JetByteTools/CoreTools/IThreadPoolWorkerThreadFactory.h"
#include "JetByteTools/CoreTools/IThreadPoolWorkerThreadCallback.h"

#include "JetByteTools/IOTools/SmartBuffer.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Core
   {
      class IIndexedOpaqueUserData;
   }

   namespace IO
   {
      class CBufferChain;
      class IAllocateBuffers;
   }

   namespace Socket
   {
      class IAllocateSequencedStreamSockets;
      class IStreamSocket;
      class IAddress;
   }
}

class CSocketServer;

///////////////////////////////////////////////////////////////////////////////
// CThreadPool
///////////////////////////////////////////////////////////////////////////////

class CThreadPool :
   private JetByteTools::Core::IThreadPoolWorkerThreadFactory,
   private JetByteTools::Core::IThreadPoolWorkerThreadCallback
{
   public:

      typedef JetByteTools::Core::CThreadPool::ThreadCount ThreadCount;

      CThreadPool(
         ThreadCount initialThreads,
         ThreadCount minThreads,
         ThreadCount maxThreads,
         ThreadCount maxDormantThreads,
         JetByteTools::Milliseconds poolMaintPeriod,
         JetByteTools::Milliseconds dispatchTimeout);

      CThreadPool(
         const CThreadPool &rhs) = delete;

      CThreadPool &operator=(
         const CThreadPool &rhs) = delete;

      void Start(
         CSocketServer &server);

      void WaitForShutdownToComplete(
         JetByteTools::Milliseconds forceTerminationAfter = INFINITE);

      void DispatchCompleteMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer);

   private :

      // Implement IThreadPoolWorkerThreadFactory

      JetByteTools::Core::IThreadPoolWorkerThreadCallback &CreateWorkerThread() override;

      void DestroyWorkerThread(
         JetByteTools::Core::IThreadPoolWorkerThreadCallback &thread) override;

      void DisposeOfQueuedItemsAfterShutdown(
         ULONG_PTR completionKey,
         DWORD numberOfBytes,
         OVERLAPPED *pOverlapped) override;

      // Implement IThreadPoolWorkerThreadCallback

      bool Initialise() override;

      void Process(
         ULONG_PTR completionKey,
         DWORD numberOfBytes,
         OVERLAPPED *pOverlapped) override;

      void Shutdown() override;

      void OnProcessException(
         const JetByteTools::Core::_tstring & /*message*/) override {}

      /// Called if a thread is terminated due to an exception.

      void OnThreadTerminationException(
         const JetByteTools::Core::_tstring & /*message*/) override {}

      CSocketServer *m_pServer;

      JetByteTools::Core::CThreadPool m_threadPool;
};

#endif // OPEN_SSL_ECHO_SERVER_THREAD_POOL_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: ThreadPool.h
///////////////////////////////////////////////////////////////////////////////
