///////////////////////////////////////////////////////////////////////////////
// File: BufferHandleAllocator.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2009 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "BufferHandleAllocator.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::IIndexedOpaqueUserData;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::IMonitorBufferAllocation;

///////////////////////////////////////////////////////////////////////////////
// CBufferHandleAllocator
///////////////////////////////////////////////////////////////////////////////

CBufferHandleAllocator::CBufferHandleAllocator(
   const BufferCount maxFreeBuffers,
   IAllocateBuffers &bufferAllocator,
   const bool log)
   :  JetByteTools::IO::CBufferHandleAllocator(*this, maxFreeBuffers, bufferAllocator),
      m_log(log)
{

}

CBufferHandleAllocator::~CBufferHandleAllocator()
{
   // Must ensure that we have been flushed before we call the base class's
   // destructor otherwise it may try and call into us (as a monitor) when
   // it deletes buffers...

   Flush();
}

void CBufferHandleAllocator::OnBufferCreated()
{
   if (m_log)
   {
      Output(_T("OnBufferCreated [Handle]"));
   }
}

void CBufferHandleAllocator::OnBufferAllocated(
   IIndexedOpaqueUserData & /*userData*/)
{
   if (m_log)
   {
      Output(_T("OnBufferAllocated [Handle]"));
   }
}

void CBufferHandleAllocator::OnBufferAllocationContention()
{

}

void CBufferHandleAllocator::OnBufferReleaseContention()
{

}

void CBufferHandleAllocator::OnBufferReleased(
   IIndexedOpaqueUserData & /*userData*/)
{
   if (m_log)
   {
      Output(_T("OnBufferReleased [Handle]"));
   }
}

void CBufferHandleAllocator::OnBufferDestroyed()
{
   if (m_log)
   {
      Output(_T("OnBufferDestroyed [Handle]"));
   }
}

///////////////////////////////////////////////////////////////////////////////
// End of file: BufferHandleAllocator.cpp
///////////////////////////////////////////////////////////////////////////////
